@@ -1,9 +1,5 @@
-import sys
-
-if sys.version_info < (3, 9):
-    raise ImportError("`corgy._corgy` requires Python 3.9 or higher.")
-
 import argparse
+import sys
 from collections import defaultdict
 from collections.abc import Sequence as AbstractSequence
 from contextlib import suppress
@@ -11,7 +7,7 @@ from functools import partial
 from typing import (
     Any,
     Callable,
-    Literal,
+    Dict,
     NamedTuple,
     Optional,
     Sequence,
@@ -19,7 +15,14 @@ from typing import (
     TypeVar,
     Union,
 )
-from warnings import warn
+
+if sys.version_info >= (3, 9):
+    from typing import Literal
+else:
+    from typing_extensions import Literal
+
+if sys.version_info >= (3, 10):
+    from types import UnionType
 
 from ._helpfmt import CorgyHelpFormatter
 
@@ -32,6 +35,50 @@ __all__ = ("Corgy", "corgyparser")
 _T = TypeVar("_T", bound="Corgy")
 
 
+class BooleanOptionalAction(argparse.Action):
+    # :meta private:
+    # Backport of `argparse.BooleanOptionalAction` from Python 3.9.
+    # Taken almost verbatim from `CPython/Lib/argparse.py`.
+    def __init__(self, option_strings, dest, *args, **kwargs):
+
+        _option_strings = []
+        for option_string in option_strings:
+            _option_strings.append(option_string)
+
+            if option_string.startswith("--"):
+                option_string = "--no-" + option_string[2:]
+                _option_strings.append(option_string)
+
+        super().__init__(
+            option_strings=_option_strings, dest=dest, nargs=0, *args, **kwargs
+        )
+
+    def __call__(self, parser, namespace, values, option_string=None):
+        if option_string in self.option_strings:
+            setattr(namespace, self.dest, not option_string.startswith("--no-"))
+
+
+def _is_union_type(t) -> bool:
+    """Check if the argument is a union type."""
+    if sys.version_info >= (3, 10):
+        # This checks for the `|` based syntax introduced in Python 3.10.
+        p310_check = t.__class__ is UnionType
+    else:
+        p310_check = False
+    return p310_check or (hasattr(t, "__origin__") and t.__origin__ is Union)
+
+
+def _is_sequence_type(t) -> bool:
+    """Check if the argument is a sequence type."""
+    if t is Sequence or t is AbstractSequence:
+        return True
+    if hasattr(t, "__origin__") and (
+        t.__origin__ is Sequence or t.__origin__ is AbstractSequence
+    ):
+        return True
+    return False
+
+
 class _CorgyMeta(type):
     """Metaclass for `Corgy`.
 
@@ -44,7 +91,10 @@ class _CorgyMeta(type):
     __slots__ = ()
 
     def __new__(cls, name, bases, namespace, **kwds):
-        namespace["__slots__"] = []
+        if "__slots__" not in namespace:
+            namespace["__slots__"] = []
+        else:
+            namespace["__slots__"] = list(namespace["__slots__"])
         if "__annotations__" not in namespace:
             return super().__new__(cls, name, bases, namespace, **kwds)
 
@@ -52,8 +102,10 @@ class _CorgyMeta(type):
         namespace["__flags"] = {}
         for var_name, var_ano in namespace["__annotations__"].items():
             # Check for name conflicts.
-            if f"_{name.lstrip('_')}__{var_name}" in (
-                namespace | namespace["__annotations__"]
+            _mangled_name = f"_{name.lstrip('_')}__{var_name}"
+            if (
+                _mangled_name in namespace
+                or _mangled_name in namespace["__annotations__"]
             ):
                 raise TypeError(
                     f"cannot use name `__{var_name}`: internal clash with `{var_name}`"
@@ -105,16 +157,20 @@ class _CorgyMeta(type):
             )
             namespace["__slots__"].append(f"__{var_name}")
 
+        namespace["__slots__"] = tuple(namespace["__slots__"])
+
         # Store custom parsers in a dict.
         namespace["__parsers"] = {}
         for _, v in namespace.items():
             if not isinstance(v, _CorgyParser):
                 continue
-            var_name = v.var_name
-            if (var_name in namespace) and isinstance(namespace[var_name], property):
-                namespace["__parsers"][var_name] = v.fparse
-            else:
-                raise TypeError(f"invalid target for corgyparser: {v.var_name}")
+            for var_name in v.var_names:
+                if (var_name in namespace) and isinstance(
+                    namespace[var_name], property
+                ):
+                    namespace["__parsers"][var_name] = v.fparse
+                else:
+                    raise TypeError(f"invalid target for corgyparser: {var_name}")
 
         return super().__new__(cls, name, bases, namespace, **kwds)
 
@@ -138,9 +194,6 @@ class _CorgyMeta(type):
 class Corgy(metaclass=_CorgyMeta):
     """Base class for collections of variables.
 
-    Note:
-        This class is only available on Python 3.9 or higher.
-
     To create a command line interface, subclass `Corgy`, and declare your arguments
     using type annotations::
 
@@ -153,22 +206,46 @@ class Corgy(metaclass=_CorgyMeta):
 
         a = A()
         a.x = 1
+        a.y  # AttributeError (y is not set)
         a.y = a.x + 1.1
 
-    Note that the class's `__init__` method only accepts keyword arguments. Refer to
-    the documentation of `Corgy.__init__` for more information.
+    Note that the class's `__init__` method only accepts keyword arguments, and ignores
+    arguments without a corresponding attribute. The following are all valid::
 
-    For command line parsing, `x` and `y` are added to an `ArgumentParser` object with
-    the appropriate arguments passed to `ArgumentParser.add_argument`. This is roughly
-    equivalent to::
+        A(x=1, y=2.1)
+        A(x=1, z=3)  # y is not set, and z is ignored
+        A(**{"x": 1, "y": 2.1, "z": 3})
+
+    For command line parsing, the `add_args_to_parser` class method can be used to add
+    arguments to an `ArgumentParser` object. Refer to the method's documentation for
+    more details. `A.add_args_to_parser(parser)` is roughly equivalent to::
 
-        parser = ArgumentParser()
         parser.add_argument("--x", type=int, required=True)
         parser.add_argument("--y", type=float, required=True)
 
+    `Corgy` classes have their `__slots__` attribute set to the annotated arguments.
+    So, if you want to use additional instance variables not tracked by `Corgy`, define
+    them (and only them) in the `__slots__` attribute::
+
+        class A(Corgy):
+            __slots__ = ("x",)
+            y: float
+
+        a = A()
+        a.y = 1  # `Corgy` variable
+        a.x = 2  # custom variable
+
+    To allow arbitrary instance variables, add `__dict__` to `__slots__`. Names added
+    through custom `__slots__` are not processed by `Corgy`, and will not be added to
+    `ArgumentParser` objects by the class methods.
+
     `Corgy` recognizes a number of special annotations, which are used to control how
     the argument is parsed.
 
+    Note:
+        If any of the following annotations are unavilable in the Python version being
+        used, you can import them from `typing_extension` (which is available on PyPI).
+
     **Annotated**:
     `typing.Annotated` can be used to add a help message::
 
@@ -192,6 +269,7 @@ class Corgy(metaclass=_CorgyMeta):
     `typing.Optional` can be used to mark an argument as optional::
 
         x: Optional[int]
+        x: int | None  # Python 3.10+ (can also use `Optional`)
 
     Another way to mark an argument as optional is to provide a default value::
 
@@ -213,8 +291,8 @@ class Corgy(metaclass=_CorgyMeta):
 
     **Sequence**
     `collections.abc.Sequence` can be used to specify that an argument accepts multiple
-    space-separated values. `typing.Sequence` can also be used, but is not recommended
-    as it is deprecated since Python 3.9.
+    space-separated values. On Python versions below 3.9, `typing.Sequence` must be
+    used instead.
 
     There are a few different ways to use `Sequence`, each resulting in different
     conditions for the parser. The simplest case is a plain sequence::
@@ -230,14 +308,16 @@ class Corgy(metaclass=_CorgyMeta):
     `--x` in the command line. After parsing, `x` will be a `list`. To denote an
     optional sequence, use `Optional[Sequence[...]]`.
 
+    The sequence length can be controlled by the arguments to `Sequence`. However, this
+    feature is only available in Python 3.9 and above, since `typing.Sequence` only
+    accepts a single argument.
+
     To specify that a sequence must be non-empty, use::
 
         x: Sequence[int, ...]
 
     This will result in `nargs` being set to `+` in the call to
-    `ArgumentParser.add_argument`. Using this syntax **requires**
-    `collections.abc.Sequence`, since `typing.Sequence` does not accept `...` as an
-    argument.
+    `ArgumentParser.add_argument`.
 
     Finally, you can specify a fixed length sequence::
 
@@ -280,8 +360,7 @@ class Corgy(metaclass=_CorgyMeta):
         x: A
 
     **Bool**
-    `bool` types (when not in a sequence) are converted to
-    `argparse.BooleanOptionalAction`::
+    `bool` types (when not in a sequence) are converted to a pair of options::
 
         class A(Corgy):
             arg: bool
@@ -317,10 +396,30 @@ class Corgy(metaclass=_CorgyMeta):
     instance of `A`, and set as the `grp` property of `B`. Note that groups will ignore
     any custom flags when computing the prefix; elements within the group will use
     custom flags, but because they are prefixed with `--`, they will not be positional.
+
+    If initializing a `Corgy` class with `__init__`, arguments for groups can be passed
+    with their names prefixed with the group name and a colon::
+
+        class C(Corgy):
+            x: int
+
+        class D(Corgy):
+            x: int
+            c: C
+
+        d = D(**{"x": 1, "c:x": 2})
+        d.x  # 1
+        d.c  # C(x=2)
     """
 
     @classmethod
-    def add_args_to_parser(cls, parser: argparse.ArgumentParser, name_prefix: str = ""):
+    def add_args_to_parser(
+        cls,
+        parser: argparse.ArgumentParser,
+        name_prefix: str = "",
+        make_group: bool = False,
+        group_help: Optional[str] = None,
+    ):
         """Add arguments for this class to the given parser.
 
         Args:
@@ -328,7 +427,14 @@ class Corgy(metaclass=_CorgyMeta):
             name_prefix: Prefix for argument names (default: empty string). Arguments
                 will be named `--<name-prefix>:<var-name>`. If custom flags are present,
                 `--<name-prefix>:<flag>` will be used instead (one for each flag).
+            make_group: If `True`, the arguments will be added to a group within the
+                parser, and `name_prefix` will be used as the group name.
+            group_help: Help text for the group. Ignored if `make_group` is `False`.
         """
+        base_parser = parser
+        if make_group:
+            parser = parser.add_argument_group(name_prefix, group_help)  # type: ignore
+
         for (var_name, var_type) in getattr(cls, "__annotations__").items():
             var_flags = getattr(cls, "__flags").get(
                 var_name, [f"--{var_name.replace('_', '-')}"]
@@ -342,20 +448,29 @@ class Corgy(metaclass=_CorgyMeta):
             else:
                 var_dest = var_name
 
+            if not any(_flag.startswith("-") for _flag in var_flags):
+                var_flags = [var_name]
+                var_positional = True
+            elif all(_flag.startswith("-") for _flag in var_flags):
+                var_positional = False
+            else:
+                raise TypeError(
+                    f"inconsistent positional/optional flags for {var_name}: "
+                    f"{var_flags}"
+                )
+
             var_help = getattr(cls, var_name).__doc__  # doc is stored in the property
 
             # Check if the variable is also `Corgy` type.
             if type(var_type) is type(cls):
                 # Create an argument group using `<var_type>`.
-                grp_parser = parser.add_argument_group(var_dest, var_help)
-                var_type.add_args_to_parser(grp_parser, var_dest)
+                var_type.add_args_to_parser(base_parser, var_dest, True, var_help)
                 continue
 
             # Check if the variable is optional. `<var_name>: Optional[<var_type>]` is
             # equivalent to `<var_name>: Union[<var_type>, None]`.
             if (
-                hasattr(var_type, "__origin__")
-                and var_type.__origin__ is Union
+                _is_union_type(var_type)
                 and len(var_type.__args__) == 2
                 and var_type.__args__[1] is type(None)
             ):
@@ -365,30 +480,16 @@ class Corgy(metaclass=_CorgyMeta):
                 var_base_type = var_type
                 var_required = var_name not in getattr(cls, "__defaults")
 
-            # Check if the variable has a custom parser.
-            _parsers = getattr(cls, "__parsers")
-            if var_name in _parsers:
-                var_fparse = _parsers[var_name]
-                _kwargs: dict[str, Any] = {}
-                if var_name in getattr(cls, "__flags"):
-                    # Explicitly pass `dest` if custom flags are present.
-                    _kwargs["dest"] = var_dest
-                if var_help is not None:
-                    _kwargs["help"] = var_help
-                _defaults = getattr(cls, "__defaults")
-                if var_name in _defaults:
-                    _kwargs["default"] = _defaults[var_name]
-                if var_required:
-                    _kwargs["required"] = True
-                parser.add_argument(*var_flags, type=var_fparse, **_kwargs)
-                continue
-
             # Check if the variable is a sequence.
             var_nargs: Union[int, Literal["+", "*"], None]
-            if hasattr(var_base_type, "__origin__") and (
-                var_base_type.__origin__ is Sequence
-                or var_base_type.__origin__ is AbstractSequence
-            ):
+            if _is_sequence_type(var_base_type):
+                if not hasattr(var_base_type, "__args__") or isinstance(
+                    var_base_type.__args__[0], TypeVar
+                ):
+                    raise TypeError(
+                        f"`{var_name}` is a sequence, but has no type arguments: "
+                        f"use `{var_base_type}[<types>]"
+                    )
                 if len(var_base_type.__args__) == 1:
                     var_nargs = "*"
                 elif (
@@ -447,13 +548,20 @@ class Corgy(metaclass=_CorgyMeta):
             # `--<var-name>`/`--no-<var-name>` arguments.
             var_action: Optional[Type[argparse.Action]]
             if var_base_type is bool and var_nargs is None:
-                var_action = argparse.BooleanOptionalAction
+                var_action = BooleanOptionalAction
             else:
                 var_action = None
 
+            # Check if the variable has a custom parser.
+            _parsers = getattr(cls, "__parsers")
+            if var_name in _parsers:
+                var_add_type = _parsers[var_name]
+            else:
+                var_add_type = var_base_type
+
             # Add the variable to the parser.
-            _kwargs = {}
-            if var_name in getattr(cls, "__flags"):
+            _kwargs: Dict[str, Any] = {}
+            if var_name in getattr(cls, "__flags") and not var_positional:
                 _kwargs["dest"] = var_dest
             if var_help is not None:
                 _kwargs["help"] = var_help
@@ -466,37 +574,14 @@ class Corgy(metaclass=_CorgyMeta):
             _defaults = getattr(cls, "__defaults")
             if var_name in _defaults:
                 _kwargs["default"] = _defaults[var_name]
-            if var_required:
+            if var_required and not var_positional:
                 _kwargs["required"] = True
-            parser.add_argument(*var_flags, type=var_base_type, **_kwargs)
+            with suppress(AttributeError):
+                _kwargs["metavar"] = var_base_type.__metavar__
+            parser.add_argument(*var_flags, type=var_add_type, **_kwargs)
 
     def __init__(self, **args):
-        """Initialize an instance of the class using the given arguments.
-
-        Arguments with `:` in their name are passed to group class constructors.
-        Unknown arguments are ignored. This is useful when using a custom parser with
-        non-Corgy arguments.
-
-        Example::
-
-            class C(Corgy):
-                x: int
-
-            class D(Corgy):
-                x: int
-                c: C
-
-            parser = argparse.ArgumentParser()
-            D.add_args_to_parser(parser)
-            parser.add_argument("--y", type=int)
-
-            args = parser.parse_args(["--x", "1", "--c:x", "11", "--y", "2"])
-            d = D(**vars(args))
-            d.x  # 1
-            d.c  # C(x=11)
-            args.y  # 2
-        """
-        grp_args_map: dict[str, Any] = defaultdict(dict)
+        grp_args_map: Dict[str, Any] = defaultdict(dict)
 
         for arg_name, arg_val in args.items():
             if ":" in arg_name:
@@ -514,33 +599,42 @@ class Corgy(metaclass=_CorgyMeta):
             grp_obj = grp_type(**grp_args)
             setattr(self, grp_name, grp_obj)
 
-    @classmethod
-    def new_with_args(cls: Type[_T], **args) -> _T:
-        #: :meta private:
-        warn(
-            f"this method is deprecated: use `{cls.__name__}(**args)` "
-            f"to create a new instance",
-            DeprecationWarning,
-        )
-        return cls(**args)
-
-    def __repr__(self):
+    def _str(self, f_str: Callable[..., str]) -> str:
         s = f"{self.__class__.__name__}("
         for i, arg_name in enumerate(getattr(self.__class__, "__annotations__")):
             if i != 0:
                 s = s + ", "
             s = s + f"{arg_name}="
             try:
-                _val_s = repr(getattr(self, arg_name))
+                _val_s = f_str(getattr(self, arg_name))
             except AttributeError:
                 _val_s = "<unset>"
             s = s + _val_s
         s = s + ")"
         return s
 
+    def __repr__(self) -> str:
+        return self._str(repr)
+
+    def __str__(self) -> str:
+        return self._str(str)
+
+    def as_dict(self) -> Dict[str, Any]:
+        """Return the object as a dictionary.
+
+        The returned dictionary maps attribute names to their values. Unset attributes
+        are omitted, unless they have default values. This method is not recursive, and
+        attributes which are `Corgy` instances are returned as is.
+        """
+        return {
+            arg_name: getattr(self, arg_name)
+            for arg_name in getattr(self.__class__, "__annotations__")
+            if hasattr(self, arg_name)
+        }
+
     @classmethod
     def parse_from_cmdline(
-        cls: type[_T], parser: Optional[argparse.ArgumentParser] = None, **parser_args
+        cls: Type[_T], parser: Optional[argparse.ArgumentParser] = None, **parser_args
     ) -> _T:
         """Parse an object of the class from command line arguments.
 
@@ -566,17 +660,22 @@ class _CorgyParser(NamedTuple):
     keep track of parsers.
     """
 
-    var_name: str
+    var_names: Sequence[str]
     fparse: Callable[[str], Any]
 
+    def __call__(self, s: str) -> Any:
+        return self.fparse(s)
 
-def corgyparser(var_name: str) -> Callable[[Callable[[str], Any]], _CorgyParser]:
+
+def corgyparser(
+    var_name: str,
+) -> Callable[[Union[Callable[[str], Any], _CorgyParser]], _CorgyParser]:
     """Decorate a function as a custom parser for a variable.
 
-    Note:
-        This decorator is only available on Python 3.9 or higher.
-
     To use a custom function for parsing an argument with `Corgy`, use this decorator.
+    Parsing functions must be static, and should only accept a single string argument.
+    Decorating the function with `@staticmethod` is optional, but prevents type errors.
+    `@corgyparser` must be the final decorator in the decorator chain.
 
     Args:
         var_name: The argument associated with the decorated parser.
@@ -586,8 +685,21 @@ def corgyparser(var_name: str) -> Callable[[Callable[[str], Any]], _CorgyParser]
         class A(Corgy):
             time: tuple[int, int, int]
             @corgyparser("time")
+            @staticmethod
             def parse_time(s):
                 return tuple(map(int, s.split(":")))
+
+    The `@corgyparser` decorator can be chained to use the same parser for multiple
+    arguments::
+
+        class A(Corgy):
+            x: int
+            y: int
+            @corgyparser("x")
+            @corgyparser("y")
+            @staticmethod
+            def parse_x_y(s):
+                return int(s)
     """
     if not isinstance(var_name, str):
         raise TypeError(
@@ -596,6 +708,14 @@ def corgyparser(var_name: str) -> Callable[[Callable[[str], Any]], _CorgyParser]
         )
 
     def wrapper(var_name, fparse):
-        return _CorgyParser(var_name, fparse)
+        if isinstance(fparse, _CorgyParser):
+            fparse.var_names.append(var_name)
+            return fparse
+
+        if isinstance(fparse, staticmethod):
+            fparse = fparse.__func__
+        if not callable(fparse):
+            raise TypeError("corgyparser can only decorate static functions")
+        return _CorgyParser([var_name], fparse)
 
     return partial(wrapper, var_name)
