@@ -1,23 +1,31 @@
+import argparse
 import sys
 import unittest
+from typing import Optional, Sequence
 from unittest import skipIf
+from unittest.mock import MagicMock, patch
+
+SequenceType = Sequence
 
 if sys.version_info >= (3, 9):
-    import argparse
-    from collections.abc import Sequence
-    from typing import Annotated, Literal, Optional, Sequence as SequenceType
-    from unittest.mock import MagicMock, patch
+    from collections.abc import Sequence  # pylint: disable=reimported
+    from typing import Annotated, Literal
+else:
+    from typing_extensions import Annotated
 
-    import corgy
-    from corgy import Corgy, CorgyHelpFormatter, corgyparser
+    if sys.version_info >= (3, 8):
+        from typing import Literal
+    else:
+        from typing_extensions import Literal
+
+import corgy
+from corgy import Corgy, CorgyHelpFormatter, corgyparser
+from corgy._corgy import BooleanOptionalAction
 
 
-@skipIf(sys.version_info < (3, 9), "Python 3.9 or higher needed")
 class TestCorgyMeta(unittest.TestCase):
     """Tests to check validity of classes inheriting from Corgy."""
 
-    # pylint: disable=unused-variable, unused-private-member
-
     @classmethod
     def setUpClass(cls):
         class _CorgyCls(Corgy):
@@ -79,19 +87,19 @@ class TestCorgyMeta(unittest.TestCase):
     def test_corgy_cls_raises_if_help_annotation_not_str(self):
         with self.assertRaises(TypeError):
 
-            class C1(Corgy):
+            class _(Corgy):
                 x: Annotated[int, 1]
 
     def test_corgy_cls_raises_if_flags_not_list(self):
         with self.assertRaises(TypeError):
 
-            class C1(Corgy):
+            class _(Corgy):
                 x: Annotated[int, "x help", "x"]
 
     def test_corgy_cls_raises_if_flag_list_empty(self):
         with self.assertRaises(TypeError):
 
-            class C1(Corgy):
+            class _(Corgy):
                 x: Annotated[int, "x help", []]
 
     def test_add_args_raises_if_custom_flags_on_group(self):
@@ -100,12 +108,12 @@ class TestCorgyMeta(unittest.TestCase):
             class G(Corgy):
                 x: int
 
-            class C(Corgy):
+            class _(Corgy):
                 g: Annotated[G, "group G", ["-g", "--grp"]]
 
     def test_corgy_cls_allows_dunder_defaults_as_var_name(self):
         class C(Corgy):
-            __defaults: int
+            __defaults: int  # pylint: disable=unused-private-member
             x: int = 0
 
         self.assertTrue(hasattr(C, "__defaults"))
@@ -117,15 +125,15 @@ class TestCorgyMeta(unittest.TestCase):
     def test_corgy_cls_raises_if_var_name_is_dunder_another_var(self):
         with self.assertRaises(TypeError):
 
-            class C1(Corgy):
+            class _C(Corgy):  # pylint: disable=unused-variable
                 x: int = 0
-                __x = 2
+                __x = 2  # pylint: disable=unused-private-member
 
         with self.assertRaises(TypeError):
 
-            class C2(Corgy):
+            class _C(Corgy):  # pylint: disable=unused-variable
                 x: int
-                __x: int
+                __x: int  # pylint: disable=unused-private-member
 
     def test_corgy_cls_can_have_dunder_name(self):
         self.assertTrue(hasattr(self._CorgyCls, "_CorgyCls__x1"))
@@ -143,12 +151,22 @@ class TestCorgyMeta(unittest.TestCase):
         with self.assertRaises(AttributeError):
             c.z = 0
 
-    def test_corgy_cls_has_correct_repr(self):
+    def test_corgy_cls_allows_custom_slots(self):
+        class C(Corgy):
+            __slots__ = ("y",)
+            x: int
+
+        c = C()
+        c.y = 1
+        self.assertEqual(c.y, 1)
+
+    def test_corgy_cls_has_correct_repr_str(self):
         c = self._CorgyCls()
         c.x1 = [0, 1]
         c.x2 = 2
         c.x4 = "8"
         self.assertEqual(repr(c), "_CorgyCls(x1=[0, 1], x2=2, x3=3, x4='8')")
+        self.assertEqual(str(c), "_CorgyCls(x1=[0, 1], x2=2, x3=3, x4=8)")
 
     def test_corgy_cls_repr_handles_unset_values(self):
         c = self._CorgyCls()
@@ -162,8 +180,28 @@ class TestCorgyMeta(unittest.TestCase):
         d = D(x=1, c=self._CorgyCls(x1=[0, 1], x2=2, x4="8"))
         self.assertEqual(repr(d), "D(x=1, c=_CorgyCls(x1=[0, 1], x2=2, x3=3, x4='8'))")
 
+    def test_corgy_cls_as_dict(self):
+        c = self._CorgyCls(x1=[0, 1], x2=2, x3=30, x4="40")
+        self.assertDictEqual(c.as_dict(), {"x1": [0, 1], "x2": 2, "x3": 30, "x4": "40"})
+
+    def test_corgy_cls_as_dict_uses_default_values(self):
+        c = self._CorgyCls(x1=[0, 1], x2=2, x3=30)
+        self.assertDictEqual(c.as_dict(), {"x1": [0, 1], "x2": 2, "x3": 30, "x4": "4"})
+
+    def test_corgy_cls_as_dict_ignores_unset_attrs_without_defaults(self):
+        c = self._CorgyCls(x3=30, x4="40")
+        self.assertDictEqual(c.as_dict(), {"x3": 30, "x4": "40"})
+
+    def test_corgy_cls_as_dict_handles_groups(self):
+        class D(Corgy):
+            x: int
+            c: self._CorgyCls
+
+        c = self._CorgyCls()
+        d = D(x=1, c=c)
+        self.assertDictEqual(d.as_dict(), {"x": 1, "c": c})
+
 
-@skipIf(sys.version_info < (3, 9), "Python 3.9 or higher needed")
 class TestCorgyAddArgsToParser(unittest.TestCase):
     """Tests to check that Corgy properly adds arguments to ArgumentParsers."""
 
@@ -190,6 +228,18 @@ class TestCorgyAddArgsToParser(unittest.TestCase):
             "--prefix:the-x-arg", type=int, required=True
         )
 
+    def test_add_args_handles_custom_metavar(self):
+        class T:
+            __metavar__ = "T"
+
+        class C(Corgy):
+            x: T
+
+        C.add_args_to_parser(self.parser)
+        self.parser.add_argument.assert_called_once_with(
+            "--x", type=T, required=True, metavar="T"
+        )
+
     def test_add_args_handles_plain_type_annotation(self):
         class C(Corgy):
             x: int
@@ -211,6 +261,14 @@ class TestCorgyAddArgsToParser(unittest.TestCase):
         C.add_args_to_parser(self.parser)
         self.parser.add_argument.assert_called_once_with("--x", type=int)
 
+    @skipIf(sys.version_info < (3, 10), "`|` syntax needs Python 3.10 or higher")
+    def test_add_args_handles_annotated_new_style_optional(self):
+        class C(Corgy):
+            x: int | None  # type: ignore # pylint: disable=unsupported-binary-operation
+
+        C.add_args_to_parser(self.parser)
+        self.parser.add_argument.assert_called_once_with("--x", type=int)
+
     def test_add_args_handles_annotated_optional_with_default(self):
         class C(Corgy):
             x: Optional[int] = 0
@@ -255,7 +313,16 @@ class TestCorgyAddArgsToParser(unittest.TestCase):
 
         C.add_args_to_parser(self.parser)
         self.parser.add_argument.assert_called_once_with(
-            "x", type=int, required=True, help="x help", dest="the_x_arg"
+            "the_x_arg", type=int, help="x help"
+        )
+
+    def test_add_handles_positional_flag_with_same_name(self):
+        class C(Corgy):
+            the_x_arg: Annotated[int, "x help", ["the-x-arg"]]
+
+        C.add_args_to_parser(self.parser)
+        self.parser.add_argument.assert_called_once_with(
+            "the_x_arg", type=int, help="x help"
         )
 
     def test_add_args_converts_literal_to_choices(self):
@@ -348,7 +415,7 @@ class TestCorgyAddArgsToParser(unittest.TestCase):
 
         C.add_args_to_parser(self.parser)
         self.parser.add_argument.assert_called_once_with(
-            "--x", type=bool, action=argparse.BooleanOptionalAction, required=True
+            "--x", type=bool, action=BooleanOptionalAction, required=True
         )
 
     def test_add_args_handles_default_for_bool_type(self):
@@ -357,7 +424,7 @@ class TestCorgyAddArgsToParser(unittest.TestCase):
 
         C.add_args_to_parser(self.parser)
         self.parser.add_argument.assert_called_once_with(
-            "--x", type=bool, action=argparse.BooleanOptionalAction, default=False
+            "--x", type=bool, action=BooleanOptionalAction, default=False
         )
 
     def test_add_args_does_not_convert_bool_sequence_to_action(self):
@@ -369,6 +436,19 @@ class TestCorgyAddArgsToParser(unittest.TestCase):
             "--x", type=bool, required=True, nargs="*"
         )
 
+    def test_add_args_raises_if_sequence_has_no_types(self):
+        class C(Corgy):
+            x: Sequence
+
+        class D(Corgy):
+            x: SequenceType
+
+        with self.assertRaises(TypeError):
+            C.add_args_to_parser(self.parser)
+
+        with self.assertRaises(TypeError):
+            D.add_args_to_parser(self.parser)
+
     def test_add_args_sets_nargs_to_asterisk_for_sequence_type(self):
         class C(Corgy):
             x: Sequence[int]
@@ -394,6 +474,7 @@ class TestCorgyAddArgsToParser(unittest.TestCase):
         C.add_args_to_parser(self.parser)
         self.parser.add_argument.assert_called_once_with("--x", type=int, nargs="*")
 
+    @skipIf(sys.version_info < (3, 9), "`typing.Sequence` doesn't accept multiple args")
     def test_add_args_sets_nargs_to_plus_for_non_empty_sequence_type(self):
         class C(Corgy):
             x: Sequence[int, ...]  # type: ignore
@@ -421,6 +502,7 @@ class TestCorgyAddArgsToParser(unittest.TestCase):
             "--x", type=int, nargs="*", required=True, choices=(1, 2, 3)
         )
 
+    @skipIf(sys.version_info < (3, 9), "`typing.Sequence` doesn't accept multiple args")
     def test_add_args_handles_fixed_length_sequence_with_chocies(self):
         class C(Corgy):
             x: Sequence[Literal[1, 2, 3], Literal[1, 2, 3]]
@@ -430,6 +512,7 @@ class TestCorgyAddArgsToParser(unittest.TestCase):
             "--x", type=int, nargs=2, required=True, choices=(1, 2, 3)
         )
 
+    @skipIf(sys.version_info < (3, 9), "`typing.Sequence` doesn't accept multiple args")
     def test_add_args_raises_if_fixed_length_sequence_choices_not_all_same(self):
         class C(Corgy):
             x: Sequence[Literal[1, 2, 3], Literal[1, 2]]
@@ -437,6 +520,7 @@ class TestCorgyAddArgsToParser(unittest.TestCase):
         with self.assertRaises(TypeError):
             C.add_args_to_parser(self.parser)
 
+    @skipIf(sys.version_info < (3, 9), "`typing.Sequence` doesn't accept multiple args")
     def test_add_args_handles_fixed_length_typed_sequence(self):
         class C(Corgy):
             x: Sequence[int, int, int]
@@ -446,6 +530,7 @@ class TestCorgyAddArgsToParser(unittest.TestCase):
             "--x", type=int, nargs=3, required=True
         )
 
+    @skipIf(sys.version_info < (3, 9), "`typing.Sequence` doesn't accept multiple args")
     def test_add_args_handles_length_2_typed_sequence(self):
         class C(Corgy):
             x: Sequence[int, int]
@@ -455,6 +540,7 @@ class TestCorgyAddArgsToParser(unittest.TestCase):
             "--x", type=int, nargs=2, required=True
         )
 
+    @skipIf(sys.version_info < (3, 9), "`typing.Sequence` doesn't accept multiple args")
     def test_add_args_raises_if_fixed_length_sequence_types_not_all_same(self):
         class C(Corgy):
             x: Sequence[int, str, int]
@@ -466,15 +552,16 @@ class TestCorgyAddArgsToParser(unittest.TestCase):
         class G(Corgy):
             x: int
 
-        G.add_args_to_parser = MagicMock()
-
         class C(Corgy):
             g: Annotated[G, "group G"]
 
+        grp_parser = MagicMock()
+        self.parser.add_argument_group = MagicMock(return_value=grp_parser)
+
         C.add_args_to_parser(self.parser)
         self.parser.add_argument_group.assert_called_once_with("g", "group G")
-        G.add_args_to_parser.assert_called_once_with(
-            self.parser.add_argument_group.return_value, "g"
+        grp_parser.add_argument.assert_called_once_with(
+            "--g:x", type=int, required=True
         )
 
     def test_add_args_allows_repeated_name_in_group(self):
@@ -512,6 +599,53 @@ class TestCorgyAddArgsToParser(unittest.TestCase):
             dest="the_grp:the_x_arg",
         )
 
+    def test_add_args_handles_custom_flags_of_positional_args_inside_group(self):
+        class G(Corgy):
+            the_x_arg: Annotated[int, "x help", ["x", "the-x", "the_x_arg"]]
+
+        grp_parser = argparse.ArgumentParser()
+        grp_parser.add_argument = MagicMock()
+        G.add_args_to_parser(grp_parser)
+        grp_parser.add_argument.assert_called_once_with(
+            "the_x_arg", type=int, help="x help"
+        )
+
+        class C(Corgy):
+            the_grp: G
+
+        grp_parser = MagicMock()
+        self.parser.add_argument_group.return_value = grp_parser
+
+        C.add_args_to_parser(self.parser)
+        self.parser.add_argument.assert_not_called()
+        self.parser.add_argument_group.assert_called_once_with("the_grp", None)
+        grp_parser.add_argument.assert_called_once_with(
+            "--the-grp:x",
+            "--the-grp:the-x",
+            "--the-grp:the_x_arg",
+            type=int,
+            help="x help",
+            required=True,
+            dest="the_grp:the_x_arg",
+        )
+
+    def test_add_args_makes_nested_groups_flat(self):
+        class G2(Corgy):
+            x: int
+
+        class G1(Corgy):
+            x: int
+            g2: G2
+
+        class C(Corgy):
+            x: int
+            g1: G1
+
+        C.add_args_to_parser(self.parser)
+        self.parser.add_argument.assert_called_once_with("--x", type=int, required=True)
+        self.parser.add_argument_group.assert_any_call("g1", None)
+        self.parser.add_argument_group.assert_any_call("g1:g2", None)
+
     def test_add_args_infers_correct_base_type_from_complex_type_hint(self):
         class C(Corgy):
             x: Annotated[Optional[Sequence[str]], "x"]
@@ -541,7 +675,6 @@ class TestCorgyAddArgsToParser(unittest.TestCase):
         )
 
 
-@skipIf(sys.version_info < (3, 9), "Python 3.9 or higher needed")
 class TestCorgyCmdlineParsing(unittest.TestCase):
     """Test cases to check parsing of command line arguments by Corgy."""
 
@@ -576,6 +709,15 @@ class TestCorgyCmdlineParsing(unittest.TestCase):
                 c = C.parse_from_cmdline(self.parser)
                 self.assertEqual(c.var, 1)
 
+    def test_cmdline_positional_args_are_parsed_with_custom_flags(self):
+        class C(Corgy):
+            var: Annotated[int, "x help", ["x"]]
+
+        self.parser = argparse.ArgumentParser()
+        self.parser.parse_args = lambda: self.orig_parse_args(self.parser, ["1"])
+        c = C.parse_from_cmdline(self.parser)
+        self.assertEqual(c.var, 1)
+
     def test_cmdline_parsing_handles_group_arguments(self):
         class G(Corgy):
             x: int
@@ -595,6 +737,30 @@ class TestCorgyCmdlineParsing(unittest.TestCase):
         self.assertEqual(c.g.x, 3)
         self.assertEqual(c.g.y, "four")
 
+    def test_cmdline_parsing_handles_groups_with_positional_args(self):
+        class G(Corgy):
+            the_x_var: Annotated[int, "x help", ["x", "the_x", "the-x-var"]]
+
+        grp_parser = argparse.ArgumentParser()
+        grp_parser.parse_args = lambda: self.orig_parse_args(grp_parser, ["1"])
+        g = G.parse_from_cmdline(grp_parser)
+        self.assertEqual(g.the_x_var, 1)
+
+        class C(Corgy):
+            x: int
+            g: G
+
+        for grp_flag in ["--g:x", "--g:the-x", "--g:the-x-var"]:
+            with self.subTest(grp_flag=grp_flag):
+                self.setUp()
+                self.parser.parse_args = lambda: self.orig_parse_args(
+                    self.parser,
+                    ["--x", "1", grp_flag, "2"],  # pylint: disable=cell-var-from-loop
+                )
+                c = C.parse_from_cmdline(self.parser)
+                self.assertEqual(c.x, 1)
+                self.assertEqual(c.g.the_x_var, 2)
+
     def test_cmdline_parsing_handles_nested_groups(self):
         class G1(Corgy):
             x: int
@@ -710,12 +876,9 @@ class TestCorgyCmdlineParsing(unittest.TestCase):
             _ = c.y
 
 
-@skipIf(sys.version_info < (3, 9), "Python 3.9 or higher needed")
 class TestCorgyCustomParsers(unittest.TestCase):
     """Tests to check usage of the @corgyparser decorator."""
 
-    # pylint: disable=no-self-argument, unused-variable
-
     def test_corgyparser_raises_if_not_passed_name(self):
         with self.assertRaises(TypeError):
 
@@ -726,11 +889,11 @@ class TestCorgyCustomParsers(unittest.TestCase):
     def test_corgy_raises_if_corgyparser_target_invalid(self):
         with self.assertRaises(TypeError):
 
-            class A(Corgy):
+            class _(Corgy):
                 x: int
 
                 @corgyparser("y")
-                def parsex(s: str):  # type: ignore
+                def parsex(s: str):  # type: ignore # pylint: disable=no-self-argument
                     return 0
 
     def test_add_args_handles_corgyparser(self):
@@ -738,7 +901,7 @@ class TestCorgyCustomParsers(unittest.TestCase):
             x: Annotated[int, "x"]
 
             @corgyparser("x")
-            def parsex(s: str):  # type: ignore
+            def parsex(s: str):  # type: ignore # pylint: disable=no-self-argument
                 return 0
 
         parser = argparse.ArgumentParser()
@@ -753,7 +916,7 @@ class TestCorgyCustomParsers(unittest.TestCase):
             x: int = 1
 
             @corgyparser("x")
-            def parsex(s: str):  # type: ignore
+            def parsex(s: str):  # type: ignore # pylint: disable=no-self-argument
                 return 0
 
         parser = argparse.ArgumentParser()
@@ -768,7 +931,7 @@ class TestCorgyCustomParsers(unittest.TestCase):
             x: int
 
             @corgyparser("x")
-            def parsex(s: str):  # type: ignore
+            def parsex(s: str):  # type: ignore # pylint: disable=no-self-argument
                 return 0
 
         getattr(C, "__parsers")["x"] = MagicMock()
@@ -784,7 +947,7 @@ class TestCorgyCustomParsers(unittest.TestCase):
             x: int
 
             @corgyparser("x")
-            def parsex(s: str):  # type: ignore
+            def parsex(s: str):  # type: ignore # pylint: disable=no-self-argument
                 return -1
 
         parser = argparse.ArgumentParser()
@@ -793,3 +956,80 @@ class TestCorgyCustomParsers(unittest.TestCase):
 
         args = C.parse_from_cmdline(parser)
         self.assertEqual(args.x, -1)
+
+    def test_corgyparser_allows_decorating_staticmethod(self):
+        class C(Corgy):
+            x: int
+
+            @corgyparser("x")
+            @staticmethod
+            def parsex(s: str):
+                return 0
+
+        parser = argparse.ArgumentParser()
+        orig_parse_args = argparse.ArgumentParser.parse_args
+        parser.parse_args = lambda: orig_parse_args(parser, ["--x", "test"])
+
+        c = C.parse_from_cmdline(parser)
+        self.assertEqual(c.x, 0)
+
+    def test_corgyparser_raises_if_decorating_non_staticmethod(self):
+        with self.assertRaises(TypeError):
+
+            class _(Corgy):
+                x: int
+
+                @corgyparser("x")
+                @classmethod
+                def parsex(cls, s: str):
+                    return 0
+
+    def test_corgyparser_functions_are_callable(self):
+        class C(Corgy):
+            x: int
+            y: int
+
+            @corgyparser("x")
+            @staticmethod
+            def parsex(s: str):
+                return 0
+
+            @corgyparser("y")
+            def parsey(s: str):  # type: ignore # pylint: disable=no-self-argument
+                return 1
+
+        self.assertEqual(C.parsex("x"), 0)
+        self.assertEqual(C.parsey("y"), 1)
+
+    def test_corgyparser_decorators_can_be_chained(self):
+        class C(Corgy):
+            x: int
+            y: int
+
+            @corgyparser("x")
+            @corgyparser("y")
+            @staticmethod
+            def parsexy(s: str):
+                return int(s)
+
+        self.assertEqual(C.parsexy("1"), 1)
+        self.assertEqual(C.parsexy("2"), 2)
+
+    def test_add_args_with_custom_parser_uses_custom_metavar(self):
+        class T:
+            __metavar__ = "T"
+
+        class C(Corgy):
+            x: T
+
+            @corgyparser("x")
+            @staticmethod
+            def parsex(s: str):
+                return 0
+
+        parser = argparse.ArgumentParser()
+        parser.add_argument = MagicMock()
+        C.add_args_to_parser(parser)
+        parser.add_argument.assert_called_once_with(
+            "--x", type=C.parsex.fparse, required=True, metavar="T"
+        )
