@@ -1,28 +1,21 @@
-import sys
 from argparse import ArgumentParser, SUPPRESS
 from unittest import skipIf, TestCase
 from unittest.mock import Mock, patch
 
 from corgy import CorgyHelpFormatter
+from corgy._corgy import BooleanOptionalAction
 from corgy._helpfmt import _ColorHelper
 
-if sys.version_info >= (3, 9):
-    from argparse import BooleanOptionalAction
-
 _COLOR_HELPER = _ColorHelper(skip_tty_check=True)
 _CRAYONS = _COLOR_HELPER.crayons
 
 # Shortcuts for color functions to make ground truths in assert statements concise.
 _M = lambda s: _COLOR_HELPER.colorize(s, CorgyHelpFormatter.color_metavars)
 _K = lambda s: _COLOR_HELPER.colorize(s, CorgyHelpFormatter.color_keywords)
-_C = lambda s: _COLOR_HELPER.colorize(repr(s), CorgyHelpFormatter.color_choices)
-_D = lambda s: _COLOR_HELPER.colorize(repr(s), CorgyHelpFormatter.color_defaults)
+_C = lambda s: _COLOR_HELPER.colorize(s, CorgyHelpFormatter.color_choices)
+_D = lambda s: _COLOR_HELPER.colorize(s, CorgyHelpFormatter.color_defaults)
 _O = lambda s: _COLOR_HELPER.colorize(s, CorgyHelpFormatter.color_options)
 
-# Versions of `_C`/`_D` which do not call `repr` on the object.
-_Cs = lambda s: _COLOR_HELPER.colorize(s, CorgyHelpFormatter.color_choices)
-_Ds = lambda s: _COLOR_HELPER.colorize(s, CorgyHelpFormatter.color_defaults)
-
 # Make outputs independent of terminal width.
 CorgyHelpFormatter.output_width = 80
 CorgyHelpFormatter.max_help_position = 80
@@ -69,7 +62,9 @@ class TestCorgyHelpFormatterAPI(TestCase):
             color_metavars="BLUE",
             color_options="yellow",
         ):
-            parser = ArgumentParser(formatter_class=CorgyHelpFormatter, add_help=False)
+            parser = ArgumentParser(
+                formatter_class=CorgyHelpFormatter, add_help=False, usage=SUPPRESS
+            )
             parser.add_argument(
                 "--x", type=int, choices=[1, 2], help="x help", default=1
             )
@@ -97,7 +92,9 @@ class TestCorgyHelpFormatterAPI(TestCase):
             marker_choices_end=" ) ",
             marker_choices_sep="|",
         ):
-            parser = ArgumentParser(formatter_class=CorgyHelpFormatter, add_help=False)
+            parser = ArgumentParser(
+                formatter_class=CorgyHelpFormatter, add_help=False, usage=SUPPRESS
+            )
             parser.add_argument("-x", "--x", type=int, choices=[1, 2])
 
             self.assertEqual(
@@ -107,11 +104,16 @@ class TestCorgyHelpFormatterAPI(TestCase):
     def test_corgy_help_formatter_handles_changing_output_width(self):
         CorgyHelpFormatter.use_colors = False
         with patch.object(CorgyHelpFormatter, "output_width", 10):
-            parser = ArgumentParser(formatter_class=CorgyHelpFormatter, add_help=False)
+            parser = ArgumentParser(
+                formatter_class=CorgyHelpFormatter, add_help=False, prog=""
+            )
             parser.add_argument("--x", type=int, help="x help")
 
             self.assertEqual(
                 parser.format_help(),
+                "usage:\n"
+                "  [--x\n"
+                "  int]\n\n"
                 "options:\n"
                 "  --x int\n"
                 "      x\n"
@@ -124,7 +126,7 @@ class TestCorgyHelpFormatterAPI(TestCase):
     def test_corgy_help_formatter_handles_changing_max_help_position(self):
         CorgyHelpFormatter.use_colors = False
         with patch.multiple(CorgyHelpFormatter, output_width=100, max_help_position=10):
-            parser = ArgumentParser(formatter_class=CorgyHelpFormatter)
+            parser = ArgumentParser(formatter_class=CorgyHelpFormatter, usage=SUPPRESS)
             parser.add_argument(
                 "--x", type=int, metavar="A LONG METAVAR", help="x help"
             )
@@ -133,29 +135,55 @@ class TestCorgyHelpFormatterAPI(TestCase):
                 parser.format_help(),
                 # options:
                 #   -h/--help
-                #       show this help message and exit (optional)
+                #       show this help message and exit
                 #   --x A LONG METAVAR
                 #       x help (optional)
                 "options:\n"
                 "  -h/--help\n"
-                "      show this help message and exit (optional)\n"
+                "      show this help message and exit\n"
                 "  --x A LONG METAVAR\n"
                 "      x help (optional)\n",
             )
 
+    def test_corgy_help_formatter_handles_changing_show_full_help(self):
+        CorgyHelpFormatter.use_colors = False
+        with patch.object(CorgyHelpFormatter, "show_full_help", False):
+            parser = ArgumentParser(
+                formatter_class=CorgyHelpFormatter,
+                add_help=False,
+                usage="this shouldn't show",
+            )
+            parser.add_argument("--x", type=int, help="x help", required=True)
+            parser.add_argument("--y", type=int, help="y help", choices=(1, 2))
+            parser.add_argument("--z", type=int, help="z help", default=0)
+
+            self.assertEqual(
+                parser.format_help(),
+                # options:
+                #   --x int  x help
+                #   --y int  y help (optional)
+                #   --z int  z help (default: 0)
+                "options:\n"
+                "  --x int  x help\n"
+                "  --y int  y help (optional)\n"
+                "  --z int  z help (default: 0)\n",
+            )
+
     @skipIf(_CRAYONS is None, "`crayons` package not found")
     def test_corgy_help_formatter_consistent_on_repeat_usage(self):
         CorgyHelpFormatter.use_colors = True
-        parser = ArgumentParser(formatter_class=CorgyHelpFormatter)
+        parser = ArgumentParser(formatter_class=CorgyHelpFormatter, prog="")
         parser.add_argument("--x", type=int, choices=[1, 2])
 
         desired_output = (
+            # usage: [-h] [--x int]
+            #
             # options:
-            #   -h/--help  show this help message and exit (optional)
+            #   -h/--help  show this help message and exit
             #   --x int    ([1/2] optional)
+            f"usage: [-h] [--x int]\n\n"
             f"options:\n"
-            f"  {_O('-h')}/{_O('--help')}  show this help message and exit "
-            f"({_K('optional')})\n"
+            f"  {_O('-h')}/{_O('--help')}  show this help message and exit\n"
             f"  {_O('--x')} {_M('int')}    ([{_C(1)}/{_C(2)}] {_K('optional')})\n"
         )
 
@@ -163,30 +191,76 @@ class TestCorgyHelpFormatterAPI(TestCase):
         self.assertEqual(parser.format_help(), desired_output)
 
         # Test with a new parser.
-        parser = ArgumentParser(formatter_class=CorgyHelpFormatter)
+        parser = ArgumentParser(formatter_class=CorgyHelpFormatter, prog="")
         parser.add_argument("--x", type=int, choices=[1, 2])
         self.assertEqual(parser.format_help(), desired_output)
 
-    def test_corgy_help_formatter_does_not_construct_usage_if_none_provided(self):
-        parser = ArgumentParser(formatter_class=CorgyHelpFormatter)
-        self.assertEqual(parser.format_usage(), "")
-
-    def test_corgy_help_formatter_shows_custom_usage_iff_called_directly(self):
-        parser = ArgumentParser(
-            formatter_class=CorgyHelpFormatter, add_help=False, usage="custom usage"
-        )
-        self.assertEqual(parser.format_usage(), "usage: custom usage\n")
-        self.assertEqual(parser.format_help(), "")
-
     @skipIf(_CRAYONS is None, "`crayons` package not found")
     def test_corgy_help_formatter_raises_if_using_invalid_color(self):
         with patch.object(CorgyHelpFormatter, "color_metavars", "ELUB"):
-            parser = ArgumentParser(formatter_class=CorgyHelpFormatter)
+            parser = ArgumentParser(formatter_class=CorgyHelpFormatter, usage=SUPPRESS)
             parser.add_argument("--x", type=str)
             with self.assertRaises(ValueError):
                 parser.format_help()
 
 
+class TestCorgyHelpFormatterHelpActions(TestCase):
+    """Tests to check the help actions in `CorgyHelpFormatter`."""
+
+    def setUp(self):
+        CorgyHelpFormatter.use_colors = False
+        self.parser = ArgumentParser(
+            formatter_class=CorgyHelpFormatter, add_help=False, usage=SUPPRESS
+        )
+        self.parser.print_help = Mock()
+        self.parser.exit = Mock()
+
+    def tearDown(self):
+        CorgyHelpFormatter.show_full_help = True
+
+    def test_corgy_help_formatter_short_help_action(self):
+        self.parser.add_argument(
+            "-h", nargs=0, action=CorgyHelpFormatter.ShortHelpAction
+        )
+        self.parser.parse_args(["-h"])
+        self.parser.print_help.assert_called_once()
+        self.parser.exit.assert_called_once()
+        self.assertEqual(CorgyHelpFormatter.show_full_help, False)
+
+    def test_corgy_help_formatter_full_help_action(self):
+        self.parser.add_argument(
+            "-h", nargs=0, action=CorgyHelpFormatter.FullHelpAction
+        )
+        self.parser.parse_args(["-h"])
+        self.parser.print_help.assert_called_once()
+        self.parser.exit.assert_called_once()
+        self.assertEqual(CorgyHelpFormatter.show_full_help, True)
+
+    def test_corgy_help_formatter_add_short_full_helps(self):
+        self.parser.add_argument = Mock()
+        CorgyHelpFormatter.add_short_full_helps(
+            self.parser,
+            short_help_flags=("-h", "--helpshort"),
+            full_help_flags=("-H", "--helpfull"),
+            short_help_msg="show short help",
+            full_help_msg="show full help",
+        )
+        self.parser.add_argument.assert_any_call(
+            "-h",
+            "--helpshort",
+            nargs=0,
+            action=CorgyHelpFormatter.ShortHelpAction,
+            help="show short help",
+        )
+        self.parser.add_argument.assert_any_call(
+            "-H",
+            "--helpfull",
+            nargs=0,
+            action=CorgyHelpFormatter.FullHelpAction,
+            help="show full help",
+        )
+
+
 @skipIf(_CRAYONS is None, "`crayons` package not found")
 class TestCorgyHelpFormatterSingleArgs(TestCase):
     """Tests to check formatting of single arguments."""
@@ -194,7 +268,9 @@ class TestCorgyHelpFormatterSingleArgs(TestCase):
     def setUp(self):
         _COLOR_HELPER.crayons = _CRAYONS
         CorgyHelpFormatter.use_colors = True
-        self.parser = ArgumentParser(formatter_class=CorgyHelpFormatter, add_help=False)
+        self.parser = ArgumentParser(
+            formatter_class=CorgyHelpFormatter, add_help=False, usage=SUPPRESS
+        )
         self.maxDiff = None  # color codes can lead to very long diffs
 
     def _get_arg_help(self, *args, **kwargs):
@@ -296,7 +372,6 @@ class TestCorgyHelpFormatterSingleArgs(TestCase):
                 f"  {_O('+++x')} {_M('str')}  x help ({_K('optional')})",
             )
 
-    @skipIf(sys.version_info < (3, 9), "`BooleanOptionalAction` not available")
     def test_corgy_help_formatter_handles_boolean_optional_action(self):
         self.assertEqual(
             self._get_arg_help(
@@ -361,9 +436,9 @@ class TestCorgyHelpFormatterSingleArgs(TestCase):
             return s
 
         self.assertEqual(
-            self._get_arg_help("--x", type=custom_type),
+            self._get_arg_help("--x", type=custom_type, default="x"),
             #   --x custom_type  (optional)
-            f"  {_O('--x')} {_M('custom_type')}  ({_K('optional')})",
+            f"  {_O('--x')} {_M('custom_type')}  ({_K('default')}: {_D('x')})",
         )
 
     def test_corgy_help_formatter_handles_custom_metavar(self):
@@ -481,26 +556,25 @@ class TestCorgyHelpFormatterSingleArgs(TestCase):
                 # This is awful.
                 #   --x str  x help ([
                 f"  {_O('--x')} {_M('str')}  x help (["
-                # 'a'/'b'/
+                # a/b/
                 + _C("a") + "/" + _C("b") + "/"
-                # '--x str/'
-                + _Cs("'--x") + " " + _Cs("str'") + "/"
-                # '[\'a\'/"b"]'/
-                + _Cs("'[\\'a\\'/\"b\"]'") + "/"
-                # "default: 'a'"/
-                + _Cs('"default:') + " " + _Cs("'a'\"") + "/"
-                # '--x str  x help
-                + _Cs("'--x") + " " + _Cs("str") + "  " + _Cs("x") + " " + _Cs("help")
-                #  ([\'a\'/"b"]
-                + " " + _Cs("([\\'a\\'/\"b\"]")
-                #  default: \'a\')}
-                + " " + _Cs("default:") + " " + _Cs("\\'a\\')'") + "] "
-                # default: 'a'
+                # --x str/
+                + _C("--x") + " " + _C("str") + "/"
+                # ['a'/"b"]/
+                + _C("['a'/\"b\"]") + "/"
+                # default: 'a'/
+                + _C("default:") + " " + _C("'a'") + "/"
+                # --x str  x help
+                + _C("--x") + " " + _C("str") + "  " + _C("x") + " " + _C("help")
+                #  (['a'/"b"]
+                + " " + _C("(['a'/\"b\"]")
+                #  default: 'a')}]
+                + " " + _C("default:") + " " + _C("'a')") + "] "
+                # default: a
                 + f"{_K('default')}: {_D('a')})",
             )
 
     def test_corgy_help_formatter_handles_long_choice(self):
-        _q = "'"  # can't use escapes inside f-strings
         with patch.multiple(CorgyHelpFormatter, output_width=15, max_help_position=5):
             self.assertEqual(
                 self._get_arg_help(
@@ -510,21 +584,21 @@ class TestCorgyHelpFormatterSingleArgs(TestCase):
                     default="a",
                 ),
                 #   --x str
-                #       (['a'/'su
-                #       percalifr
-                #       agilistic
-                #       expialido
-                #       cious cho
-                #       ice'/'b']
+                #       ([a/super
+                #       califragi
+                #       listicexp
+                #       ialidocio
+                #       us
+                #       choice/b]
                 #       default:
-                #       'a')
+                #       a)
                 f"  {_O('--x')} {_M('str')}\n"
-                f"      ([{_C('a')}/{_Cs(_q+'su')}\n"
-                f"      {_Cs('percalifr')}\n"
-                f"      {_Cs('agilistic')}\n"
-                f"      {_Cs('expialido')}\n"
-                f"      {_Cs('cious')} {_Cs('cho')}\n"
-                f"      {_Cs('ice'+_q)}/{_C('b')}]\n"
+                f"      ([{_C('a')}/{_C('super')}\n"
+                f"      {_C('califragi')}\n"
+                f"      {_C('listicexp')}\n"
+                f"      {_C('ialidocio')}\n"
+                f"      {_C('us')}\n"
+                f"      {_C('choice')}/{_C('b')}]\n"
                 f"      {_K('default')}:\n"
                 f"      {_D('a')})",
             )
@@ -541,7 +615,7 @@ class TestCorgyHelpFormatterSingleArgs(TestCase):
 
         self.assertEqual(
             self._get_arg_help("--arg", choices=[A]),
-            f"  {_O('--arg')}  ([{_Cs('A')}] {_K('optional')})",
+            f"  {_O('--arg')}  ([{_C('A')}] {_K('optional')})",
         )
 
     def test_corgy_help_formatter_uses_name_for_default(self):
@@ -550,40 +624,20 @@ class TestCorgyHelpFormatterSingleArgs(TestCase):
 
         self.assertEqual(
             self._get_arg_help("--arg", default=A),
-            f"  {_O('--arg')}  ({_K('default')}: {_Ds('A')})",
+            f"  {_O('--arg')}  ({_K('default')}: {_D('A')})",
         )
 
     def test_corgy_help_formatter_uses_custom_choice_strs(self):
         class A:
-            @staticmethod
-            def __corgy_fmt_choice__(obj):
-                return f"A:{obj}"
-
-        self.assertEqual(
-            self._get_arg_help("--arg", type=A, choices=[1, "a"]),
-            f"  {_O('--arg')} {_M('A')}  ([{_Cs('A:1')}/{_Cs('A:a')}] "
-            f"{_K('optional')})",
-        )
-
-    def test_corgy_help_formatter_uses_custom_choice_strs_from_type_factory(self):
-        class AF:
-            __metavar__ = "AF"
-
             def __init__(self, x):
                 self.x = x
 
-            def __corgy_fmt_choice__(self, obj):
-                return f"{self}:{obj}"
-
-            def __str__(self):
-                return str(self.x)
-
-            def __call__(self, string):
-                return string
+            def __repr__(self):
+                return f"A:{self.x}"
 
         self.assertEqual(
-            self._get_arg_help("--arg", type=AF("A"), choices=[1, "a"]),
-            f"  {_O('--arg')} {_M('AF')}  ([{_Cs('A:1')}/{_Cs('A:a')}] "
+            self._get_arg_help("--arg", type=A, choices=[A(1), A("a")]),
+            f"  {_O('--arg')} {_M('A')}  ([{_C('A:1')}/{_C('A:a')}] "
             f"{_K('optional')})",
         )
 
@@ -595,7 +649,7 @@ class TestCorgyHelpFormatterMultiArgs(TestCase):
     def setUp(self):
         _COLOR_HELPER.crayons = _CRAYONS
         CorgyHelpFormatter.use_colors = True
-        self.parser = ArgumentParser(formatter_class=CorgyHelpFormatter)
+        self.parser = ArgumentParser(formatter_class=CorgyHelpFormatter, usage=SUPPRESS)
         self.maxDiff = None
 
     def test_corgy_help_formatter_handles_multi_arg_alignment(self):
@@ -606,13 +660,12 @@ class TestCorgyHelpFormatterMultiArgs(TestCase):
         self.assertEqual(
             self.parser.format_help(),
             # options:
-            #   -h/--help       show this help message and exit (optional)
+            #   -h/--help       show this help message and exit
             #   --x int         x help
             #   -y/--why float  (required)
-            #   -z str          z help (default: 'z')
+            #   -z str          z help (default: z)
             f"options:\n"
-            f"  {_O('-h')}/{_O('--help')}       show this help message and exit "
-            f"({_K('optional')})\n"
+            f"  {_O('-h')}/{_O('--help')}       show this help message and exit\n"
             f"  {_O('--x')} {_M('int')}         x help ({_K('optional')})\n"
             f"  {_O('-y')}/{_O('--why')} {_M('float')}  ({_K('required')})\n"
             f"  {_O('-z')} {_M('str')}          z help ({_K('default')}: {_D('z')})\n",
@@ -627,7 +680,6 @@ class TestCorgyHelpFormatterMultiArgs(TestCase):
                 #   -h/--help      show this
                 #                  help message
                 #                  and exit
-                #                  (optional)
                 #   -x/--ex float  helphelphelph
                 #                  elphelphelphe
                 #                  lphelphelphel
@@ -636,7 +688,6 @@ class TestCorgyHelpFormatterMultiArgs(TestCase):
                 f"  {_O('-h')}/{_O('--help')}      show this\n"
                 f"                 help message\n"
                 f"                 and exit\n"
-                f"                 ({_K('optional')})\n"
                 f"  {_O('-x')}/{_O('--ex')} {_M('float')}  helphelphelph\n"
                 f"                 elphelphelphe\n"
                 f"                 lphelphelphel\n"
@@ -650,12 +701,12 @@ class TestCorgyHelpFormatterMultiArgs(TestCase):
                 self.parser.format_help(),
                 # options:
                 #   -h/--help
-                #       show this help message and exit (optional)
+                #       show this help message and exit
                 #   -x/--ex float
                 #       helphelphelphelphelphelphelphelphelphelp (optional)
                 f"options:\n"
                 f"  {_O('-h')}/{_O('--help')}\n"
-                f"      show this help message and exit ({_K('optional')})\n"
+                f"      show this help message and exit\n"
                 f"  {_O('-x')}/{_O('--ex')} {_M('float')}\n"
                 f"      helphelphelphelphelphelphelphelphelphelp ({_K('optional')})\n",
             )
@@ -675,7 +726,7 @@ class TestCorgyHelpFormatterMultiArgs(TestCase):
             #   arg        arg help
             #
             # options:
-            #   -h/--help  show this help message and exit (optional)
+            #   -h/--help  show this help message and exit
             #   --x str    x help (optional)
             #
             # group 1:
@@ -689,8 +740,7 @@ class TestCorgyHelpFormatterMultiArgs(TestCase):
             f"  {_O('arg')}        arg help\n"
             f"\n"
             f"options:\n"
-            f"  {_O('-h')}/{_O('--help')}  show this help message and exit "
-            f"({_K('optional')})\n"
+            f"  {_O('-h')}/{_O('--help')}  show this help message and exit\n"
             f"  {_O('--x')} {_M('str')}    x help ({_K('optional')})\n"
             f"\n"
             f"group 1:\n"
@@ -712,21 +762,86 @@ class TestCorgyHelpFormatterMultiArgs(TestCase):
         self.assertEqual(
             self.parser.format_help(),
             # positional arguments:
-            #   CMD        sub commands (['x'/'y'])
+            #   CMD        sub commands ([x/y])
             #
             # options:
-            #   -h/--help  show this help message and exit (optional)
+            #   -h/--help  show this help message and exit
             #   --arg str  (optional)
             f"positional arguments:\n"
             f"  {_O('CMD')}        sub commands ([{_C('x')}/{_C('y')}])\n"
             f"\n"
             f"options:\n"
-            f"  {_O('-h')}/{_O('--help')}  show this help message and exit "
-            f"({_K('optional')})\n"
+            f"  {_O('-h')}/{_O('--help')}  show this help message and exit\n"
             f"  {_O('--arg')} {_M('str')}  ({_K('optional')})\n",
         )
 
 
+class TestCorgyHelpFormatterUsage(TestCase):
+    """Tests to check formatting of usage string."""
+
+    def setUp(self):
+        self.parser = ArgumentParser(
+            formatter_class=CorgyHelpFormatter, add_help=False, prog=""
+        )
+
+    def test_corgy_help_formatter_usage_with_positional_arg(self):
+        self.parser.add_argument("arg", type=int)
+        self.assertEqual(
+            self.parser.format_usage(),
+            # usage: arg
+            "usage: arg\n",
+        )
+
+    def test_corgy_help_formatter_usage_with_required_arg(self):
+        self.parser.add_argument("--arg", type=str, required=True)
+        self.assertEqual(
+            self.parser.format_usage(),
+            # usage: --arg str
+            "usage: --arg str\n",
+        )
+
+    def test_corgy_help_formatter_usage_with_optional_arg(self):
+        self.parser.add_argument("--arg", type=str)
+        self.assertEqual(
+            self.parser.format_usage(),
+            # usage: [--arg str]
+            "usage: [--arg str]\n",
+        )
+
+    def test_corgy_help_formatter_usage_with_group(self):
+        self.parser.add_argument("--arg", type=str)
+        grp_parser = self.parser.add_argument_group("grp")
+        grp_parser.add_argument("--grp:arg", type=str)
+        self.assertEqual(
+            self.parser.format_usage(),
+            # usage: [--arg str] [--grp:arg str]
+            "usage: [--arg str] [--grp:arg str]\n",
+        )
+
+    def test_corgy_help_formatter_usage_with_choices(self):
+        self.parser.add_argument("--arg", type=int, choices=(1, 2))
+        self.assertEqual(
+            self.parser.format_usage(),
+            # usage: [--arg int]
+            "usage: [--arg int]\n",
+        )
+
+    def test_corgy_help_formatter_always_shows_usage_when_called_explicitly(self):
+        with patch.object(CorgyHelpFormatter, "show_full_help", False):
+            self.parser.add_argument("--arg", type=str)
+            self.assertEqual(
+                self.parser.format_usage(),
+                # usage: [--arg str]
+                "usage: [--arg str]\n",
+            )
+            self.assertEqual(
+                self.parser.format_help(),
+                # options:
+                #   --arg str  (optional)
+                f"options:\n" f"  {_O('--arg')} {_M('str')}  ({_K('optional')})\n",
+            )
+
+
 class _NoColorTestMeta(type):
     """Metaclass to create versions of test classes that don't use colors."""
 
@@ -754,7 +869,9 @@ class TestCorgyHelpFormatterSingleArgsNoColor(
     def setUp(self):
         _COLOR_HELPER.crayons = None
         CorgyHelpFormatter.use_colors = False
-        self.parser = ArgumentParser(formatter_class=CorgyHelpFormatter, add_help=False)
+        self.parser = ArgumentParser(
+            formatter_class=CorgyHelpFormatter, add_help=False, usage=SUPPRESS
+        )
 
 
 class TestCorgyHelpFormatterMultiArgsNoColor(
@@ -765,4 +882,4 @@ class TestCorgyHelpFormatterMultiArgsNoColor(
     def setUp(self):
         _COLOR_HELPER.crayons = None
         CorgyHelpFormatter.use_colors = False
-        self.parser = ArgumentParser(formatter_class=CorgyHelpFormatter)
+        self.parser = ArgumentParser(formatter_class=CorgyHelpFormatter, usage=SUPPRESS)
